﻿using Microsoft.EntityFrameworkCore;
using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.SearchModels;
using SecuritySystemContracts.StoragesContracts;
using SecuritySystemContracts.ViewModels;
using SecuritySystemDatabaseImplement.Models;

namespace SecuritySystemDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            return GetFilteredList(model).FirstOrDefault();
        }

        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            var orders = GetFullList();
            if (model.Id != null)
            {
                orders = orders.Where(x => x.Id == model.Id.Value).ToList();
            }
            if (model.Status != null)
            {
                orders = orders.Where(x => x.Status == model.Status.Value).ToList();
            }
            if (model.ImplementerId != null)
            {
                orders = orders.Where(x => x.ImplementerId == model.ImplementerId.Value).ToList();
            }
            if (model.DateFrom != null)
            {
                orders = orders.Where(x => x.DateCreate >= model.DateFrom.Value).ToList();
            }
            if (model.DateTo != null)
            {
                orders = orders.Where(x => x.DateCreate <= model.DateTo.Value).ToList();
            }
            if (model.ClientId != null)
            {
                orders = orders.Where(x => x.ClientId == model.ClientId.Value).ToList();
            }
            return orders;
        }

        public List<OrderViewModel> GetFullList()
        {
            using var context = new SecuritySystemDatabase();
            return context.Orders
                .Include(x => x.Secure)
                .Include(x => x.Client)
                .Include(x => x.Implementer)
                .Select(x => x.GetViewModel)
                .ToList();
        }

        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var newOrder = Order.Create(model);
            if (newOrder == null)
            {
                return null;
            }
            using var context = new SecuritySystemDatabase();
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return GetFullList().FirstOrDefault(x => x.Id == newOrder.Id);
        }

        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new SecuritySystemDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return GetFullList().FirstOrDefault(x => x.Id == model.Id);
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new SecuritySystemDatabase();
            var element = context.Orders.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
