﻿using Microsoft.EntityFrameworkCore;
using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.ViewModels;
using SecuritySystemDataModels.Models;
using System.ComponentModel.DataAnnotations;

namespace SecuritySystemDatabaseImplement.Models
{
    [PrimaryKey("MessageId")]
    public class MessageInfo : IMessageInfoModel
    {
        public string MessageId { get; set; } = string.Empty;
        public int? ClientId { get; set; }
        public virtual Client? Client { get; set; }
        [Required]
        public string SenderName { get; set; } = string.Empty;
        [Required]
        public DateTime DateDelivery { get; set; } = DateTime.Now;
        [Required]
        public string Subject { get; set; } = string.Empty;
        [Required]
        public string Body { get; set; } = string.Empty;

        public static MessageInfo? Create(MessageInfoBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new MessageInfo()
            {
                MessageId = model.MessageId,
                ClientId = model.ClientId,
                SenderName = model.SenderName,
                DateDelivery = model.DateDelivery,
                Subject = model.Subject,
                Body = model.Body,
            };
        }
        public static MessageInfo Create(MessageInfoViewModel model)
        {
            return new MessageInfo
            {
                MessageId = model.MessageId,
                ClientId = model.ClientId,
                SenderName = model.SenderName,
                DateDelivery = model.DateDelivery,
                Subject = model.Subject,
                Body = model.Body,
            };
        }
        public void Update(MessageInfoBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            MessageId = model.MessageId;
            ClientId = model.ClientId;
            SenderName = model.SenderName;
            DateDelivery = model.DateDelivery;
            Subject = model.Subject;
            Body = model.Body;
        }
        public MessageInfoViewModel GetViewModel => new()
        {
            MessageId = MessageId,
            ClientId = ClientId,
            SenderName = SenderName,
            DateDelivery = DateDelivery,
            Subject = Subject,
            Body = Body,
        };
    }
}
