﻿using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.SearchModels;
using SecuritySystemContracts.StoragesContracts;
using SecuritySystemContracts.ViewModels;
using SecuritySystemFileImplement.Models;

namespace SecuritySystemFileImplement.Implements
{
    public class ClientStorage : IClientStorage
    {
        private readonly DataFileSingleton source;
        public ClientStorage()
        {
            source = DataFileSingleton.GetInstance();
        }
        public List<ClientViewModel> GetFullList()
        {
            return source.Clients.Select(x => x.GetViewModel).ToList();
        }
        public List<ClientViewModel> GetFilteredList(ClientSearchModel model)
        {
            var clients = source.Clients
            .Select(x => x.GetViewModel)
            .ToList();
            if (model.Id.HasValue)
            {
                clients = clients.Where(x => x.Id == model.Id.Value).ToList();
            }
            if (model.Email != null)
            {
                clients = clients.Where(x => x.Email == model.Email).ToList();
            }
            if (model.Password != null)
            {
                clients = clients.Where(x => x.Password == model.Password).ToList();
            }
            if (model.ClientFIO != null)
            {
                clients = clients.Where(x => x.ClientFIO == model.ClientFIO).ToList();
            }
            return clients;
        }
        public ClientViewModel? GetElement(ClientSearchModel model)
        {
            return GetFilteredList(model).FirstOrDefault();
        }
        public ClientViewModel? Insert(ClientBindingModel model)
        {
            model.Id = source.Clients.Count > 0 ? source.Clients.Max(x => x.Id) + 1 : 1;
            var newClient = Client.Create(model);
            if (newClient == null)
            {
                return null;
            }
            source.Clients.Add(newClient);
            source.SaveClients();
            return newClient.GetViewModel;
        }
        public ClientViewModel? Update(ClientBindingModel model)
        {
            var client = source.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (client == null)
            {
                return null;
            }
            client.Update(model);
            source.SaveClients();
            return client.GetViewModel;
        }
        public ClientViewModel? Delete(ClientBindingModel model)
        {
            var element = source.Clients.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                source.Clients.Remove(element);
                source.SaveClients();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
