﻿using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.ViewModels;
using SecuritySystemDataModels.Models;
using System.Xml.Linq;

namespace SecuritySystemFileImplement.Models
{
    public class Implementer : IImplementerModel
    {
        public string ImplementerFIO { get; private set; } = string.Empty;
        public string Password { get; private set; } = string.Empty;
        public int WorkExperience { get; private set; }
        public int Qualification { get; private set; }
        public int Id { get; private set; }
        public static Implementer? Create(ImplementerBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Implementer()
            {
                Id = model.Id,
                Qualification = model.Qualification,
                Password = model.Password,
                ImplementerFIO = model.ImplementerFIO,
                WorkExperience = model.WorkExperience
            };
        }
        public static Implementer? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new Implementer()
            {
                Id = Convert.ToInt32(element.Attribute("Id")!.Value),
                Qualification = Convert.ToInt32(element.Element("Qualification")!.Value),
                Password = Convert.ToString(element.Element("Password")!.Value),
                ImplementerFIO = Convert.ToString(element.Element("ImplementerFIO")!.Value),
                WorkExperience = Convert.ToInt32(element.Element("WorkExperience")!.Value)
            };
        }
        public void Update(ImplementerBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            Qualification = model.Qualification;
            Password = model.Password;
            ImplementerFIO = model.ImplementerFIO;
            WorkExperience = model.WorkExperience;
        }
        public ImplementerViewModel GetViewModel => new()
        {
            Id = Id,
            Qualification = Qualification,
            Password = Password,
            ImplementerFIO = ImplementerFIO,
            WorkExperience = WorkExperience
        };
        public XElement GetXElement => new("Сlient",
            new XAttribute("Id", Id),
            new XElement("Qualification", Qualification),
            new XElement("Password", Password),
            new XElement("ImplementerFIO", ImplementerFIO),
            new XElement("WorkExperience", WorkExperience)
        );
    }
}
