﻿using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.ViewModels;
using SecuritySystemDataModels.Models;
using System.Reflection;
using System.Xml.Linq;

namespace SecuritySystemFileImplement.Models
{
    public class MessageInfo : IMessageInfoModel
    {
        public string MessageId { get; set; } = string.Empty;
        public int? ClientId { get; set; }
        public string SenderName { get; set; } = string.Empty;
        public DateTime DateDelivery { get; set; } = DateTime.Now;
        public string Subject { get; set; } = string.Empty;
        public string Body { get; set; } = string.Empty;

        public static MessageInfo? Create(MessageInfoBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new MessageInfo()
            {
                MessageId = model.MessageId,
                ClientId = model.ClientId,
                SenderName = model.SenderName,
                DateDelivery = model.DateDelivery,
                Subject = model.Subject,
                Body = model.Body,
            };
        }
        public static MessageInfo? Create(XElement element)
        {
            if (element == null)
            {
                return null;
            }
            return new MessageInfo()
            {
                MessageId = element.Attribute("MessageId")!.Value,
                ClientId = Convert.ToInt32(element.Element("ClientId")!.Value),
                SenderName = element.Element("SenderName")!.Value,
                DateDelivery = Convert.ToDateTime(element.Element("DateDelivery")!.Value),
                Subject = element.Element("Subject")!.Value,
                Body = element.Element("Body")!.Value,
            };
        }
        public void Update(MessageInfoBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            MessageId = model.MessageId;
            ClientId = model.ClientId;
            SenderName = model.SenderName;
            DateDelivery = model.DateDelivery;
            Subject = model.Subject;
            Body = model.Body;
        }
        public MessageInfoViewModel GetViewModel => new()
        {
            MessageId = MessageId,
            ClientId = ClientId,
            SenderName = SenderName,
            DateDelivery = DateDelivery,
            Subject = Subject,
            Body = Body,
        };
        public XElement GetXElement => new("MessageInfo",
            new XAttribute("MessageId", MessageId),
            new XElement("ClientId", ClientId.ToString()),
            new XElement("SenderName", SenderName),
            new XElement("DateDelivery", DateDelivery.ToString()),
            new XElement("Subject", Subject),
            new XElement("Body", Body)
        );
    }
}
